﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.ComponentModel;

namespace CustomControl
{
    [ToolboxData("<{0}:VirtualPagerGridView runat=server></{0}:VirtualPagerGridView>")]
    public class VirtualPagerGridView : GridView
    {
        private int? virtualRecordCount;
        /// <summary>
        /// Gets or Sets the Total Records for pagination,
        /// its important to set the value of Total
        /// Records BEFORE binding the data to the grid.
        /// </summary>
        public int VirtualRecordCount
        {
            get
            {
                if (virtualRecordCount == null)
                {
                    if (ViewState["VirtualRecordCount"] == null)
                    {
                        VirtualRecordCount = PageIndex;
                    }
                    else
                    {
                        VirtualRecordCount = (int)ViewState["VirtualRecordCount"];
                    }
                }
                return virtualRecordCount.Value;
            }

            set
            {
                virtualRecordCount = value;
                ViewState["VirtualRecordCount"] = value;
            }
        }

        private bool allowCustomPaging = false;
        /// <summary>
        /// 设置是否允许自定义分页
        /// </summary>
        [Bindable(true)]
        [Category("Page")]
        [DefaultValue("false")]
        public bool AllowCustomPaging
        {
            get
            {
                if (ViewState["IsCustomPaging"] != null)
                {
                    allowCustomPaging = (bool)ViewState["IsCustomPaging"];
                }

                return allowCustomPaging;
            }

            set
            {
                allowCustomPaging = value;
                ViewState["IsCustomPaging"] = value;
            }
        }

        private bool allowCacheDatatSource = true;
        /// <summary>
        /// 设置是否允许缓存DataSource到ViewState
        /// </summary>
        [Bindable(true)]
        [Category("Page")]
        [DefaultValue("false")]
        public bool AllowCacheDatatSource
        {
            get
            {
                if (ViewState["AllowCacheDatatSource"] != null)
                {
                    allowCacheDatatSource = (bool)ViewState["AllowCacheDatatSource"];
                }

                return allowCacheDatatSource;
            }

            set
            {
                allowCacheDatatSource = value;
                ViewState["AllowCacheDatatSource"] = value;
            }
        }

        /// <summary>
        /// 返回自定义分页数量
        /// </summary>
        public override int PageCount
        {
            get
            {
                if (!AllowCustomPaging)
                {
                    return base.PageCount;
                }
                else
                {
                    return Convert.ToInt32(Math.Ceiling((decimal)VirtualRecordCount / (decimal)this.PageSize));
                }
            }
        }

        /// <summary>
        /// 返回当前分页页码
        /// </summary>
        public override int PageIndex
        {
            get
            {
                if (!AllowCustomPaging)
                {
                    return base.PageIndex;
                }
                else
                {
                    if (ViewState["VirtualPageIndex"] == null)
                    {
                        return base.PageIndex;
                    }
                    else
                    {
                        return (int)ViewState["VirtualPageIndex"];
                    }
                }
            }

            set
            {
                base.PageIndex = value;
                ViewState["VirtualPageIndex"] = value;
            }
        }

        /// <summary>
        /// 从ViewState获取DataSource
        /// </summary>
        public object CacheDataSource
        {
            get
            {
                if (this.AllowCacheDatatSource && this.EnableViewState)
                {
                    return ViewState["CacheDataSource"];
                }

                return null;
            }
        }

        /// <summary>
        /// 重写DataSource，如果 AllowCacheDatatSource为True 和 允许ViewState，则将全部数据保存到ViewState
        /// </summary>
        public override object DataSource
        {
            get
            {
                return base.DataSource;
            }
            set
            {
                if (this.AllowCacheDatatSource && this.EnableViewState)
                {
                    ViewState["CacheDataSource"] = value;
                }

                base.DataSource = value;
            }
        }

        /// <summary>
        /// Override it to avoid the popup message of not handling this event and
        /// to reset the selected index again.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPageIndexChanging(GridViewPageEventArgs e)
        {
            PageIndex = e.NewPageIndex;
            this.SelectedIndex = -1;

            base.OnPageIndexChanging(e);
        }
    }
}
