﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Data;

/// <summary>
/// 所有页面的基类
/// </summary>
public class BasePage : System.Web.UI.Page
{
    public BasePage()
    {
    }

    #region 分页相关方法
    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        if (!IsPostBack)
        {
            GridView_DataBind();
        }
    }
    /// <summary>
    /// 实现GridView数据绑定的虚方法
    /// 在具体的页面类中重写这个方法，在PageTurn方法中就会调用重写的方法，以实现分页后的数据重新绑定
    /// </summary>
    protected virtual void GridView_DataBind()
    {
    }

    /// <summary>
    /// 分页页码跳转
    /// </summary>
    /// <param name="sender">跳转按钮</param>
    /// <param name="e"></param>
    protected void GridView_PageTurn(object sender, EventArgs e)
    {
        System.Web.UI.WebControls.Button btnGoPage;
        CustomControl.VirtualPagerGridView GridView1;
        System.Web.UI.WebControls.TextBox txtGoPage;
        try
        {
            btnGoPage = (System.Web.UI.WebControls.Button)sender;
            GridView1 = (CustomControl.VirtualPagerGridView)btnGoPage.NamingContainer.Parent.Parent;
            txtGoPage = (System.Web.UI.WebControls.TextBox)GridView1.BottomPagerRow.FindControl("txtGoPage");
        }
        catch
        {
            ClientScript.RegisterStartupScript(this.Page.GetType(), "error", "<script>alert('页码输入框和跳转按钮都必须在GridView的分页模板中！');</script>");
            return;
        }

        int pageIndex = 1;
        bool goSuccess = false;

        if (!string.IsNullOrEmpty(txtGoPage.Text.Trim()))
        {
            if (int.TryParse(txtGoPage.Text.Trim(), out pageIndex))
            {
                pageIndex--;

                if (pageIndex >= 0 && pageIndex < GridView1.PageCount)
                {
                    goSuccess = true;

                    GridView1.PageIndex = pageIndex;
                    GridView_DataBind();
                }
            }
        }

        if (!goSuccess)
        {
            ClientScript.RegisterStartupScript(this.Page.GetType(), "error", "<script>alert('无效的页码！');</script>");
            return;
        }
    }

    /// <summary>
    /// 页码改变时触发
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void GridView_PageIndexChanging(object sender, System.Web.UI.WebControls.GridViewPageEventArgs e)
    {
        CustomControl.VirtualPagerGridView GridView1 = sender as CustomControl.VirtualPagerGridView;

        // 如果允许将DataSource数据保存到ViewState中，则从ViewState获取数据源
        if (GridView1.AllowCacheDatatSource && GridView1.EnableViewState)
        {
            GridView1.DataSource = GridView1.CacheDataSource;
            GridView1.DataBind();
        }
        else
        {
            GridView_DataBind();
        }
    }

    /// <summary>
    /// GridView数据绑定完毕之后触发，显示记录数
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void GridView_DataBound(object sender, EventArgs e)
    {
        //获取当前GridView
        CustomControl.VirtualPagerGridView GridView1 = sender as CustomControl.VirtualPagerGridView;

        if (GridView1.BottomPagerRow == null)
        {
            return;
        }

        // 总是显示分页行
        GridView1.BottomPagerRow.Visible = true;

        System.Web.UI.WebControls.Label lblRecorCount = (System.Web.UI.WebControls.Label)GridView1.BottomPagerRow.FindControl("lblRecorCount");

        // 如果是自定义分页，则从VirtualRecordCount属性获取数据记录总数，如果不是则从DataSource获取
        if (GridView1.AllowCustomPaging)
        {
            lblRecorCount.Text = GridView1.VirtualRecordCount.ToString();
        }
        else
        {
            if (GridView1.DataSource == null)
            {
                return;
            }

            //根据数据类型，动态获取绑定的数据源的记录数
            if (GridView1.DataSource.GetType() == typeof(DataView))
            {
                lblRecorCount.Text = ((DataView)GridView1.DataSource).Count.ToString();
            }
            else if (GridView1.DataSource.GetType() == typeof(DataTable))
            {
                lblRecorCount.Text = ((DataTable)GridView1.DataSource).Rows.Count.ToString();
            }
            else if (GridView1.DataSource.GetType() == typeof(DataSet))
            {
                lblRecorCount.Text = ((DataSet)GridView1.DataSource).Tables[0].Rows.Count.ToString();
            }
            else if (GridView1.DataSource is Array)
            {
                lblRecorCount.Text = ((Array)GridView1.DataSource).Length.ToString();
            }
            else if (GridView1.DataSource.GetType() is System.Collections.IList)
            {
                lblRecorCount.Text = ((System.Collections.IList)GridView1.DataSource).Count.ToString();
            }
            else if (GridView1.DataSource.GetType() is System.Collections.ICollection)
            {
                lblRecorCount.Text = ((System.Collections.ICollection)GridView1.DataSource).Count.ToString();
            }
            else if (GridView1.DataSource.GetType().FullName.StartsWith("System.Collections.Generic.List`1"))
            {
                lblRecorCount.Text = ((System.Collections.IList)GridView1.DataSource).Count.ToString();
            }
            else
            {
                ClientScript.RegisterStartupScript(this.Page.GetType(), "error", "<script>alert('绑定到GridView的数据类型未知！：" + GridView1.DataSource.GetType() + "！');</script>");
            }
        }
    }
    #endregion
}